@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2012, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -111,9 +111,44 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
-
 #include "aslcompiler.h"
 #include "aslcompiler.y.h"
 #include "amlcode.h"
@@ -151,27 +186,31 @@ AnMapArgTypeToBtype (
 
     switch (ArgType)
     {
-
     /* Simple types */
 
     case ARGI_ANYTYPE:
+
         return (ACPI_BTYPE_OBJECTS_AND_REFS);
 
     case ARGI_PACKAGE:
+
         return (ACPI_BTYPE_PACKAGE);
 
     case ARGI_EVENT:
+
         return (ACPI_BTYPE_EVENT);
 
     case ARGI_MUTEX:
+
         return (ACPI_BTYPE_MUTEX);
 
     case ARGI_DDBHANDLE:
         /*
          * DDBHandleObject := SuperName
-         * ACPI_BTYPE_REFERENCE: Index reference as parameter of Load/Unload
+         * ACPI_BTYPE_REFERENCE_OBJECT:
+         *      Index reference as parameter of Load/Unload
          */
-        return (ACPI_BTYPE_DDB_HANDLE | ACPI_BTYPE_REFERENCE);
+        return (ACPI_BTYPE_DDB_HANDLE | ACPI_BTYPE_REFERENCE_OBJECT);
 
     /* Interchangeable types */
     /*
@@ -183,58 +222,86 @@ AnMapArgTypeToBtype (
     case ARGI_BUFFER:
     case ARGI_BUFFER_OR_STRING:
     case ARGI_COMPUTEDATA:
+
         return (ACPI_BTYPE_COMPUTE_DATA);
 
     /* References */
 
     case ARGI_INTEGER_REF:
+
         return (ACPI_BTYPE_INTEGER);
 
     case ARGI_OBJECT_REF:
+
         return (ACPI_BTYPE_ALL_OBJECTS);
 
     case ARGI_DEVICE_REF:
+
         return (ACPI_BTYPE_DEVICE_OBJECTS);
 
     case ARGI_REFERENCE:
-        return (ACPI_BTYPE_REFERENCE);
+
+        return (ACPI_BTYPE_NAMED_REFERENCE); /* Name or Namestring */
 
     case ARGI_TARGETREF:
+
+        /*
+         * Target operand for most math and logic operators.
+         * Package objects not allowed as target.
+         */
+        return (ACPI_BTYPE_COMPUTE_DATA | ACPI_BTYPE_DEBUG_OBJECT |
+            ACPI_BTYPE_REFERENCE_OBJECT);
+
+    case ARGI_STORE_TARGET:
+
+        /* Special target for Store(), includes packages */
+
+        return (ACPI_BTYPE_DATA | ACPI_BTYPE_DEBUG_OBJECT |
+            ACPI_BTYPE_REFERENCE_OBJECT);
+
     case ARGI_FIXED_TARGET:
     case ARGI_SIMPLE_TARGET:
+
         return (ACPI_BTYPE_OBJECTS_AND_REFS);
 
     /* Complex types */
 
     case ARGI_DATAOBJECT:
-
         /*
          * Buffer, string, package or reference to a Op -
          * Used only by SizeOf operator
          */
         return (ACPI_BTYPE_STRING | ACPI_BTYPE_BUFFER |
-            ACPI_BTYPE_PACKAGE | ACPI_BTYPE_REFERENCE);
+            ACPI_BTYPE_PACKAGE | ACPI_BTYPE_REFERENCE_OBJECT);
 
     case ARGI_COMPLEXOBJ:
 
         /* Buffer, String, or package */
 
-        return (ACPI_BTYPE_STRING | ACPI_BTYPE_BUFFER | ACPI_BTYPE_PACKAGE);
+        return (ACPI_BTYPE_STRING | ACPI_BTYPE_BUFFER |
+            ACPI_BTYPE_PACKAGE);
 
     case ARGI_REF_OR_STRING:
-        return (ACPI_BTYPE_STRING | ACPI_BTYPE_REFERENCE);
+
+        /* Used by DeRefOf operator only */
+
+        return (ACPI_BTYPE_STRING | ACPI_BTYPE_REFERENCE_OBJECT);
 
     case ARGI_REGION_OR_BUFFER:
 
         /* Used by Load() only. Allow buffers in addition to regions/fields */
 
-        return (ACPI_BTYPE_REGION | ACPI_BTYPE_BUFFER | ACPI_BTYPE_FIELD_UNIT);
+        return (ACPI_BTYPE_REGION | ACPI_BTYPE_BUFFER |
+            ACPI_BTYPE_FIELD_UNIT);
 
     case ARGI_DATAREFOBJ:
-        return (ACPI_BTYPE_INTEGER |ACPI_BTYPE_STRING | ACPI_BTYPE_BUFFER |
-            ACPI_BTYPE_PACKAGE | ACPI_BTYPE_REFERENCE | ACPI_BTYPE_DDB_HANDLE);
+
+        /* Used by Store() only, as the source operand */
+
+        return (ACPI_BTYPE_DATA_REFERENCE | ACPI_BTYPE_REFERENCE_OBJECT);
 
     default:
+
         break;
     }
 
@@ -261,7 +328,6 @@ AnMapEtypeToBtype (
     UINT32                  Etype)
 {
 
-
     if (Etype == ACPI_TYPE_ANY)
     {
         return (ACPI_BTYPE_OBJECTS_AND_REFS);
@@ -279,31 +345,38 @@ AnMapEtypeToBtype (
         switch (Etype)
         {
         case ACPI_TYPE_INTEGER:
+
             return (ACPI_BTYPE_COMPUTE_DATA | ACPI_BTYPE_DDB_HANDLE);
 
         case ACPI_TYPE_STRING:
         case ACPI_TYPE_BUFFER:
+
             return (ACPI_BTYPE_COMPUTE_DATA);
 
         case ACPI_TYPE_PACKAGE:
+
             return (ACPI_BTYPE_PACKAGE);
 
         case ACPI_TYPE_FIELD_UNIT:
+
             return (ACPI_BTYPE_COMPUTE_DATA | ACPI_BTYPE_FIELD_UNIT);
 
         case ACPI_TYPE_BUFFER_FIELD:
+
             return (ACPI_BTYPE_COMPUTE_DATA | ACPI_BTYPE_BUFFER_FIELD);
 
         case ACPI_TYPE_DDB_HANDLE:
+
             return (ACPI_BTYPE_INTEGER | ACPI_BTYPE_DDB_HANDLE);
 
-        case ACPI_BTYPE_DEBUG_OBJECT:
+        case ACPI_TYPE_DEBUG_OBJECT:
 
             /* Cannot be used as a source operand */
 
             return (0);
 
         default:
+
             return (1 << (Etype - 1));
         }
     }
@@ -328,9 +401,10 @@ AnMapEtypeToBtype (
     case ACPI_TYPE_LOCAL_RESOURCE:
     case ACPI_TYPE_LOCAL_RESOURCE_FIELD:
 
-        return (ACPI_BTYPE_REFERENCE);
+        return (ACPI_BTYPE_REFERENCE_OBJECT);
 
     default:
+
         printf ("Unhandled encoded type: %X\n", Etype);
         return (0);
     }
@@ -360,7 +434,6 @@ AnFormatBtype (
 
 
     *Buffer = 0;
-
     if (Btype == 0)
     {
         strcat (Buffer, "NoReturnValue");
@@ -375,6 +448,7 @@ AnFormatBtype (
             {
                 strcat (Buffer, "|");
             }
+
             First = FALSE;
             strcat (Buffer, AcpiUtGetTypeName (Type));
         }
@@ -387,6 +461,7 @@ AnFormatBtype (
         {
             strcat (Buffer, "|");
         }
+
         First = FALSE;
         strcat (Buffer, "Reference");
     }
@@ -398,7 +473,7 @@ AnFormatBtype (
         {
             strcat (Buffer, "|");
         }
-        First = FALSE;
+
         strcat (Buffer, "Resource");
     }
 }
@@ -427,6 +502,12 @@ AnGetBtype (
     UINT32                  ThisNodeBtype = 0;
 
 
+    if (!Op)
+    {
+        AcpiOsPrintf ("Null Op in AnGetBtype\n");
+        return (ACPI_UINT32_MAX);
+    }
+
     if ((Op->Asl.ParseOpcode == PARSEOP_NAMESEG)     ||
         (Op->Asl.ParseOpcode == PARSEOP_NAMESTRING)  ||
         (Op->Asl.ParseOpcode == PARSEOP_METHODCALL))
@@ -434,11 +515,25 @@ AnGetBtype (
         Node = Op->Asl.Node;
         if (!Node)
         {
+            /* These are not expected to have a node at this time */
+
+            if ((Op->Asl.Parent->Asl.ParseOpcode == PARSEOP_CREATEWORDFIELD) ||
+                (Op->Asl.Parent->Asl.ParseOpcode == PARSEOP_CREATEDWORDFIELD) ||
+                (Op->Asl.Parent->Asl.ParseOpcode == PARSEOP_CREATEQWORDFIELD) ||
+                (Op->Asl.Parent->Asl.ParseOpcode == PARSEOP_CREATEBYTEFIELD) ||
+                (Op->Asl.Parent->Asl.ParseOpcode == PARSEOP_CREATEBITFIELD) ||
+                (Op->Asl.Parent->Asl.ParseOpcode == PARSEOP_CREATEFIELD)    ||
+                (Op->Asl.Parent->Asl.ParseOpcode == PARSEOP_CONDREFOF))
+            {
+                return (ACPI_UINT32_MAX - 1);
+            }
+
             DbgPrint (ASL_DEBUG_OUTPUT,
-                "No attached Nsnode: [%s] at line %u name [%s], ignoring typecheck\n",
+                "No attached Nsnode: [%s] at line %u name [%s], "
+                "ignoring typecheck. Parent [%s]\n",
                 Op->Asl.ParseOpName, Op->Asl.LineNumber,
-                Op->Asl.ExternalName);
-            return (ACPI_UINT32_MAX);
+                Op->Asl.ExternalName, Op->Asl.Parent->Asl.ParseOpName);
+            return (ACPI_UINT32_MAX - 1);
         }
 
         ThisNodeBtype = AnMapEtypeToBtype (Node->Type);
@@ -448,12 +543,6 @@ AnGetBtype (
                 "could not map type");
         }
 
-        /*
-         * Since it was a named reference, enable the
-         * reference bit also
-         */
-        ThisNodeBtype |= ACPI_BTYPE_REFERENCE;
-
         if (Op->Asl.ParseOpcode == PARSEOP_METHODCALL)
         {
             ReferencedNode = Node->Op;
@@ -471,7 +560,7 @@ AnGetBtype (
                 return (ACPI_UINT32_MAX);
             }
 
-            if (ReferencedNode->Asl.CompileFlags & NODE_METHOD_TYPED)
+            if (ReferencedNode->Asl.CompileFlags & OP_METHOD_TYPED)
             {
                 ThisNodeBtype = ReferencedNode->Asl.AcpiBtype;
             }
@@ -510,51 +599,67 @@ AnMapObjTypeToBtype (
     switch (Op->Asl.ParseOpcode)
     {
     case PARSEOP_OBJECTTYPE_BFF:        /* "BuffFieldObj" */
+
         return (ACPI_BTYPE_BUFFER_FIELD);
 
     case PARSEOP_OBJECTTYPE_BUF:        /* "BuffObj" */
+
         return (ACPI_BTYPE_BUFFER);
 
     case PARSEOP_OBJECTTYPE_DDB:        /* "DDBHandleObj" */
+
         return (ACPI_BTYPE_DDB_HANDLE);
 
     case PARSEOP_OBJECTTYPE_DEV:        /* "DeviceObj" */
+
         return (ACPI_BTYPE_DEVICE);
 
     case PARSEOP_OBJECTTYPE_EVT:        /* "EventObj" */
+
         return (ACPI_BTYPE_EVENT);
 
     case PARSEOP_OBJECTTYPE_FLD:        /* "FieldUnitObj" */
+
         return (ACPI_BTYPE_FIELD_UNIT);
 
     case PARSEOP_OBJECTTYPE_INT:        /* "IntObj" */
+
         return (ACPI_BTYPE_INTEGER);
 
     case PARSEOP_OBJECTTYPE_MTH:        /* "MethodObj" */
+
         return (ACPI_BTYPE_METHOD);
 
     case PARSEOP_OBJECTTYPE_MTX:        /* "MutexObj" */
+
         return (ACPI_BTYPE_MUTEX);
 
     case PARSEOP_OBJECTTYPE_OPR:        /* "OpRegionObj" */
+
         return (ACPI_BTYPE_REGION);
 
     case PARSEOP_OBJECTTYPE_PKG:        /* "PkgObj" */
+
         return (ACPI_BTYPE_PACKAGE);
 
     case PARSEOP_OBJECTTYPE_POW:        /* "PowerResObj" */
+
         return (ACPI_BTYPE_POWER);
 
     case PARSEOP_OBJECTTYPE_STR:        /* "StrObj" */
+
         return (ACPI_BTYPE_STRING);
 
     case PARSEOP_OBJECTTYPE_THZ:        /* "ThermalZoneObj" */
+
         return (ACPI_BTYPE_THERMAL);
 
     case PARSEOP_OBJECTTYPE_UNK:        /* "UnknownObj" */
+
         return (ACPI_BTYPE_OBJECTS_AND_REFS);
 
     default:
+
         return (0);
     }
 }
