@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2015, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -111,6 +111,42 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
 #include "acpi.h"
@@ -151,8 +187,18 @@ AcpiDbDumpMethodInfo (
     ACPI_WALK_STATE         *WalkState)
 {
     ACPI_THREAD_STATE       *Thread;
+    ACPI_NAMESPACE_NODE     *Node;
 
 
+    Node = WalkState->MethodNode;
+
+    /* There are no locals or arguments for the module-level code case */
+
+    if (Node == AcpiGbl_RootNode)
+    {
+        return;
+    }
+
     /* Ignore control codes, they are not errors */
 
     if ((Status & AE_CODE_MASK) == AE_CODE_CONTROL)
@@ -233,10 +279,10 @@ AcpiDbDecodeInternalObject (
 
     case ACPI_TYPE_STRING:
 
-        AcpiOsPrintf ("(%u) \"%.24s",
+        AcpiOsPrintf ("(%u) \"%.60s",
             ObjDesc->String.Length, ObjDesc->String.Pointer);
 
-        if (ObjDesc->String.Length > 24)
+        if (ObjDesc->String.Length > 60)
         {
             AcpiOsPrintf ("...");
         }
@@ -374,7 +420,7 @@ AcpiDbDisplayInternalObject (
 
             AcpiOsPrintf ("[%s] ", AcpiUtGetReferenceName (ObjDesc));
 
-            /* Decode the refererence */
+            /* Decode the reference */
 
             switch (ObjDesc->Reference.Class)
             {
@@ -520,8 +566,14 @@ AcpiDbDecodeLocals (
     BOOLEAN                 DisplayLocals = FALSE;
 
 
-    ObjDesc = WalkState->MethodDesc;
-    Node    = WalkState->MethodNode;
+    Node = WalkState->MethodNode;
+
+    /* There are no locals for the module-level code case */
+
+    if (Node == AcpiGbl_RootNode)
+    {
+        return;
+    }
 
     if (!Node)
     {
@@ -552,7 +604,7 @@ AcpiDbDecodeLocals (
 
     if (DisplayLocals)
     {
-        AcpiOsPrintf ("\nInitialized Local Variables for method [%4.4s]:\n",
+        AcpiOsPrintf ("\nInitialized Local Variables for Method [%4.4s]:\n",
             AcpiUtGetNodeName (Node));
 
         for (i = 0; i < ACPI_METHOD_NUM_LOCALS; i++)
@@ -568,7 +620,7 @@ AcpiDbDecodeLocals (
     else
     {
         AcpiOsPrintf (
-            "No Local Variables are initialized for method [%4.4s]\n",
+            "No Local Variables are initialized for Method [%4.4s]\n",
             AcpiUtGetNodeName (Node));
     }
 }
@@ -597,7 +649,13 @@ AcpiDbDecodeArguments (
 
 
     Node = WalkState->MethodNode;
-    ObjDesc = WalkState->MethodDesc;
+
+    /* There are no arguments for the module-level code case */
+
+    if (Node == AcpiGbl_RootNode)
+    {
+        return;
+    }
 
     if (!Node)
     {
@@ -631,7 +689,7 @@ AcpiDbDecodeArguments (
         AcpiOsPrintf (
             "Initialized Arguments for Method [%4.4s]:  "
             "(%X arguments defined for method invocation)\n",
-            AcpiUtGetNodeName (Node), ObjDesc->Method.ParamCount);
+            AcpiUtGetNodeName (Node), Node->Object->Method.ParamCount);
 
         for (i = 0; i < ACPI_METHOD_NUM_ARGS; i++)
         {
