@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2012, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -111,10 +111,44 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
-#define __UTCOPY_C__
-
 #include "acpi.h"
 #include "accommon.h"
 #include "acnamesp.h"
@@ -220,7 +254,7 @@ AcpiUtCopyIsimpleToEsimple (
 
     /* Always clear the external object */
 
-    ACPI_MEMSET (ExternalObject, 0, sizeof (ACPI_OBJECT));
+    memset (ExternalObject, 0, sizeof (ACPI_OBJECT));
 
     /*
      * In general, the external object will be the same type as
@@ -237,33 +271,30 @@ AcpiUtCopyIsimpleToEsimple (
         ExternalObject->String.Pointer = (char *) DataSpace;
         ExternalObject->String.Length  = InternalObject->String.Length;
         *BufferSpaceUsed = ACPI_ROUND_UP_TO_NATIVE_WORD (
-                            (ACPI_SIZE) InternalObject->String.Length + 1);
+            (ACPI_SIZE) InternalObject->String.Length + 1);
 
-        ACPI_MEMCPY ((void *) DataSpace,
+        memcpy ((void *) DataSpace,
             (void *) InternalObject->String.Pointer,
             (ACPI_SIZE) InternalObject->String.Length + 1);
         break;
 
-
     case ACPI_TYPE_BUFFER:
 
         ExternalObject->Buffer.Pointer = DataSpace;
         ExternalObject->Buffer.Length  = InternalObject->Buffer.Length;
         *BufferSpaceUsed = ACPI_ROUND_UP_TO_NATIVE_WORD (
-                            InternalObject->String.Length);
+            InternalObject->String.Length);
 
-        ACPI_MEMCPY ((void *) DataSpace,
+        memcpy ((void *) DataSpace,
             (void *) InternalObject->Buffer.Pointer,
             InternalObject->Buffer.Length);
         break;
 
-
     case ACPI_TYPE_INTEGER:
 
         ExternalObject->Integer.Value = InternalObject->Integer.Value;
         break;
 
-
     case ACPI_TYPE_LOCAL_REFERENCE:
 
         /* This is an object reference. */
@@ -271,7 +302,6 @@ AcpiUtCopyIsimpleToEsimple (
         switch (InternalObject->Reference.Class)
         {
         case ACPI_REFCLASS_NAME:
-
             /*
              * For namepath, return the object handle ("reference")
              * We are referring to the namespace node
@@ -290,7 +320,6 @@ AcpiUtCopyIsimpleToEsimple (
         }
         break;
 
-
     case ACPI_TYPE_PROCESSOR:
 
         ExternalObject->Processor.ProcId =
@@ -301,7 +330,6 @@ AcpiUtCopyIsimpleToEsimple (
             InternalObject->Processor.Length;
         break;
 
-
     case ACPI_TYPE_POWER:
 
         ExternalObject->PowerResource.SystemLevel =
@@ -311,7 +339,6 @@ AcpiUtCopyIsimpleToEsimple (
             InternalObject->PowerResource.ResourceOrder;
         break;
 
-
     default:
         /*
          * There is no corresponding external object type
@@ -356,34 +383,31 @@ AcpiUtCopyIelementToEelement (
     ACPI_FUNCTION_ENTRY ();
 
 
-    ThisIndex    = State->Pkg.Index;
-    TargetObject = (ACPI_OBJECT *)
-        &((ACPI_OBJECT *)(State->Pkg.DestObject))->Package.Elements[ThisIndex];
+    ThisIndex = State->Pkg.Index;
+    TargetObject = (ACPI_OBJECT *) &((ACPI_OBJECT *)
+        (State->Pkg.DestObject))->Package.Elements[ThisIndex];
 
     switch (ObjectType)
     {
     case ACPI_COPY_TYPE_SIMPLE:
-
         /*
          * This is a simple or null object
          */
         Status = AcpiUtCopyIsimpleToEsimple (SourceObject,
-                        TargetObject, Info->FreeSpace, &ObjectSpace);
+            TargetObject, Info->FreeSpace, &ObjectSpace);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
         }
         break;
 
-
     case ACPI_COPY_TYPE_PACKAGE:
-
         /*
          * Build the package object
          */
-        TargetObject->Type              = ACPI_TYPE_PACKAGE;
-        TargetObject->Package.Count     = SourceObject->Package.Count;
-        TargetObject->Package.Elements  =
+        TargetObject->Type = ACPI_TYPE_PACKAGE;
+        TargetObject->Package.Count = SourceObject->Package.Count;
+        TargetObject->Package.Elements =
             ACPI_CAST_PTR (ACPI_OBJECT, Info->FreeSpace);
 
         /*
@@ -396,17 +420,17 @@ AcpiUtCopyIelementToEelement (
          * update the buffer length counter
          */
         ObjectSpace = ACPI_ROUND_UP_TO_NATIVE_WORD (
-                            (ACPI_SIZE) TargetObject->Package.Count *
-                            sizeof (ACPI_OBJECT));
+            (ACPI_SIZE) TargetObject->Package.Count *
+            sizeof (ACPI_OBJECT));
         break;
 
-
     default:
+
         return (AE_BAD_PARAMETER);
     }
 
-    Info->FreeSpace   += ObjectSpace;
-    Info->Length      += ObjectSpace;
+    Info->FreeSpace += ObjectSpace;
+    Info->Length += ObjectSpace;
     return (Status);
 }
 
@@ -452,28 +476,28 @@ AcpiUtCopyIpackageToEpackage (
     /*
      * Free space begins right after the first package
      */
-    Info.Length      = ACPI_ROUND_UP_TO_NATIVE_WORD (sizeof (ACPI_OBJECT));
-    Info.FreeSpace   = Buffer + ACPI_ROUND_UP_TO_NATIVE_WORD (
-                                    sizeof (ACPI_OBJECT));
+    Info.Length = ACPI_ROUND_UP_TO_NATIVE_WORD (sizeof (ACPI_OBJECT));
+    Info.FreeSpace = Buffer +
+        ACPI_ROUND_UP_TO_NATIVE_WORD (sizeof (ACPI_OBJECT));
     Info.ObjectSpace = 0;
     Info.NumPackages = 1;
 
-    ExternalObject->Type             = InternalObject->Common.Type;
-    ExternalObject->Package.Count    = InternalObject->Package.Count;
-    ExternalObject->Package.Elements = ACPI_CAST_PTR (ACPI_OBJECT,
-                                            Info.FreeSpace);
+    ExternalObject->Type = InternalObject->Common.Type;
+    ExternalObject->Package.Count = InternalObject->Package.Count;
+    ExternalObject->Package.Elements =
+        ACPI_CAST_PTR (ACPI_OBJECT, Info.FreeSpace);
 
     /*
      * Leave room for an array of ACPI_OBJECTS in the buffer
      * and move the free space past it
      */
-    Info.Length    += (ACPI_SIZE) ExternalObject->Package.Count *
-                            ACPI_ROUND_UP_TO_NATIVE_WORD (sizeof (ACPI_OBJECT));
+    Info.Length += (ACPI_SIZE) ExternalObject->Package.Count *
+        ACPI_ROUND_UP_TO_NATIVE_WORD (sizeof (ACPI_OBJECT));
     Info.FreeSpace += ExternalObject->Package.Count *
-                            ACPI_ROUND_UP_TO_NATIVE_WORD (sizeof (ACPI_OBJECT));
+        ACPI_ROUND_UP_TO_NATIVE_WORD (sizeof (ACPI_OBJECT));
 
     Status = AcpiUtWalkPackageTree (InternalObject, ExternalObject,
-                AcpiUtCopyIelementToEelement, &Info);
+        AcpiUtCopyIelementToEelement, &Info);
 
     *SpaceUsed = Info.Length;
     return_ACPI_STATUS (Status);
@@ -512,7 +536,7 @@ AcpiUtCopyIobjectToEobject (
          * nested packages)
          */
         Status = AcpiUtCopyIpackageToEpackage (InternalObject,
-                        RetBuffer->Pointer, &RetBuffer->Length);
+            RetBuffer->Pointer, &RetBuffer->Length);
     }
     else
     {
@@ -520,10 +544,10 @@ AcpiUtCopyIobjectToEobject (
          * Build a simple object (no nested objects)
          */
         Status = AcpiUtCopyIsimpleToEsimple (InternalObject,
-                    ACPI_CAST_PTR (ACPI_OBJECT, RetBuffer->Pointer),
-                    ACPI_ADD_PTR (UINT8, RetBuffer->Pointer,
-                        ACPI_ROUND_UP_TO_NATIVE_WORD (sizeof (ACPI_OBJECT))),
-                    &RetBuffer->Length);
+            ACPI_CAST_PTR (ACPI_OBJECT, RetBuffer->Pointer),
+            ACPI_ADD_PTR (UINT8, RetBuffer->Pointer,
+                ACPI_ROUND_UP_TO_NATIVE_WORD (sizeof (ACPI_OBJECT))),
+            &RetBuffer->Length);
         /*
          * build simple does not include the object size in the length
          * so we add it in here
@@ -573,7 +597,7 @@ AcpiUtCopyEsimpleToIsimple (
     case ACPI_TYPE_LOCAL_REFERENCE:
 
         InternalObject = AcpiUtCreateInternalObject (
-                            (UINT8) ExternalObject->Type);
+            (UINT8) ExternalObject->Type);
         if (!InternalObject)
         {
             return_ACPI_STATUS (AE_NO_MEMORY);
@@ -586,6 +610,7 @@ AcpiUtCopyEsimpleToIsimple (
         return_ACPI_STATUS (AE_OK);
 
     default:
+
         /* All other types are not supported */
 
         ACPI_ERROR ((AE_INFO,
@@ -611,14 +636,13 @@ AcpiUtCopyEsimpleToIsimple (
             goto ErrorExit;
         }
 
-        ACPI_MEMCPY (InternalObject->String.Pointer,
-                     ExternalObject->String.Pointer,
-                     ExternalObject->String.Length);
+        memcpy (InternalObject->String.Pointer,
+            ExternalObject->String.Pointer,
+            ExternalObject->String.Length);
 
-        InternalObject->String.Length  = ExternalObject->String.Length;
+        InternalObject->String.Length = ExternalObject->String.Length;
         break;
 
-
     case ACPI_TYPE_BUFFER:
 
         InternalObject->Buffer.Pointer =
@@ -628,33 +652,34 @@ AcpiUtCopyEsimpleToIsimple (
             goto ErrorExit;
         }
 
-        ACPI_MEMCPY (InternalObject->Buffer.Pointer,
-                     ExternalObject->Buffer.Pointer,
-                     ExternalObject->Buffer.Length);
+        memcpy (InternalObject->Buffer.Pointer,
+            ExternalObject->Buffer.Pointer,
+            ExternalObject->Buffer.Length);
 
-        InternalObject->Buffer.Length  = ExternalObject->Buffer.Length;
+        InternalObject->Buffer.Length = ExternalObject->Buffer.Length;
 
         /* Mark buffer data valid */
 
         InternalObject->Buffer.Flags |= AOPOBJ_DATA_VALID;
         break;
 
-
     case ACPI_TYPE_INTEGER:
 
-        InternalObject->Integer.Value   = ExternalObject->Integer.Value;
+        InternalObject->Integer.Value = ExternalObject->Integer.Value;
         break;
 
     case ACPI_TYPE_LOCAL_REFERENCE:
 
-        /* TBD: should validate incoming handle */
+        /* An incoming reference is defined to be a namespace node */
 
-        InternalObject->Reference.Class = ACPI_REFCLASS_NAME;
-        InternalObject->Reference.Node = ExternalObject->Reference.Handle;
+        InternalObject->Reference.Class = ACPI_REFCLASS_REFOF;
+        InternalObject->Reference.Object = ExternalObject->Reference.Handle;
         break;
 
     default:
+
         /* Other types can't get here */
+
         break;
     }
 
@@ -698,7 +723,8 @@ AcpiUtCopyEpackageToIpackage (
 
     /* Create the package object */
 
-    PackageObject = AcpiUtCreatePackageObject (ExternalObject->Package.Count);
+    PackageObject = AcpiUtCreatePackageObject (
+        ExternalObject->Package.Count);
     if (!PackageObject)
     {
         return_ACPI_STATUS (AE_NO_MEMORY);
@@ -707,14 +733,14 @@ AcpiUtCopyEpackageToIpackage (
     PackageElements = PackageObject->Package.Elements;
 
     /*
-     * Recursive implementation. Probably ok, since nested external packages
-     * as parameters should be very rare.
+     * Recursive implementation. Probably ok, since nested external
+     * packages as parameters should be very rare.
      */
     for (i = 0; i < ExternalObject->Package.Count; i++)
     {
         Status = AcpiUtCopyEobjectToIobject (
-                    &ExternalObject->Package.Elements[i],
-                    &PackageElements[i]);
+            &ExternalObject->Package.Elements[i],
+            &PackageElements[i]);
         if (ACPI_FAILURE (Status))
         {
             /* Truncate package and delete it */
@@ -761,14 +787,16 @@ AcpiUtCopyEobjectToIobject (
 
     if (ExternalObject->Type == ACPI_TYPE_PACKAGE)
     {
-        Status = AcpiUtCopyEpackageToIpackage (ExternalObject, InternalObject);
+        Status = AcpiUtCopyEpackageToIpackage (
+            ExternalObject, InternalObject);
     }
     else
     {
         /*
          * Build a simple object (no nested objects)
          */
-        Status = AcpiUtCopyEsimpleToIsimple (ExternalObject, InternalObject);
+        Status = AcpiUtCopyEsimpleToIsimple (ExternalObject,
+            InternalObject);
     }
 
     return_ACPI_STATUS (Status);
@@ -815,7 +843,7 @@ AcpiUtCopySimpleObject (
         CopySize = sizeof (ACPI_NAMESPACE_NODE);
     }
 
-    ACPI_MEMCPY (ACPI_CAST_PTR (char, DestDesc),
+    memcpy (ACPI_CAST_PTR (char, DestDesc),
         ACPI_CAST_PTR (char, SourceDesc), CopySize);
 
     /* Restore the saved fields */
@@ -849,7 +877,7 @@ AcpiUtCopySimpleObject (
 
             /* Copy the actual buffer data */
 
-            ACPI_MEMCPY (DestDesc->Buffer.Pointer,
+            memcpy (DestDesc->Buffer.Pointer,
                 SourceDesc->Buffer.Pointer, SourceDesc->Buffer.Length);
         }
         break;
@@ -871,7 +899,7 @@ AcpiUtCopySimpleObject (
 
             /* Copy the actual string data */
 
-            ACPI_MEMCPY (DestDesc->String.Pointer, SourceDesc->String.Pointer,
+            memcpy (DestDesc->String.Pointer, SourceDesc->String.Pointer,
                 (ACPI_SIZE) SourceDesc->String.Length + 1);
         }
         break;
@@ -919,7 +947,7 @@ AcpiUtCopySimpleObject (
     case ACPI_TYPE_EVENT:
 
         Status = AcpiOsCreateSemaphore (ACPI_NO_UNIT_LIMIT, 0,
-                    &DestDesc->Event.OsSemaphore);
+            &DestDesc->Event.OsSemaphore);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -927,7 +955,9 @@ AcpiUtCopySimpleObject (
         break;
 
     default:
+
         /* Nothing to do for other simple objects */
+
         break;
     }
 
@@ -963,9 +993,9 @@ AcpiUtCopyIelementToIelement (
     ACPI_FUNCTION_ENTRY ();
 
 
-    ThisIndex     = State->Pkg.Index;
+    ThisIndex = State->Pkg.Index;
     ThisTargetPtr = (ACPI_OPERAND_OBJECT **)
-                        &State->Pkg.DestObject->Package.Elements[ThisIndex];
+        &State->Pkg.DestObject->Package.Elements[ThisIndex];
 
     switch (ObjectType)
     {
@@ -979,7 +1009,7 @@ AcpiUtCopyIelementToIelement (
              * This is a simple object, just copy it
              */
             TargetObject = AcpiUtCreateInternalObject (
-                                SourceObject->Common.Type);
+                SourceObject->Common.Type);
             if (!TargetObject)
             {
                 return (AE_NO_MEMORY);
@@ -1001,14 +1031,13 @@ AcpiUtCopyIelementToIelement (
         }
         break;
 
-
     case ACPI_COPY_TYPE_PACKAGE:
-
         /*
          * This object is a package - go down another nesting level
          * Create and build the package object
          */
-        TargetObject = AcpiUtCreatePackageObject (SourceObject->Package.Count);
+        TargetObject = AcpiUtCreatePackageObject (
+            SourceObject->Package.Count);
         if (!TargetObject)
         {
             return (AE_NO_MEMORY);
@@ -1025,8 +1054,8 @@ AcpiUtCopyIelementToIelement (
         *ThisTargetPtr = TargetObject;
         break;
 
-
     default:
+
         return (AE_BAD_PARAMETER);
     }
 
@@ -1065,16 +1094,16 @@ AcpiUtCopyIpackageToIpackage (
     ACPI_FUNCTION_TRACE (UtCopyIpackageToIpackage);
 
 
-    DestObj->Common.Type    = SourceObj->Common.Type;
-    DestObj->Common.Flags   = SourceObj->Common.Flags;
-    DestObj->Package.Count  = SourceObj->Package.Count;
+    DestObj->Common.Type = SourceObj->Common.Type;
+    DestObj->Common.Flags = SourceObj->Common.Flags;
+    DestObj->Package.Count = SourceObj->Package.Count;
 
     /*
      * Create the object array and walk the source package tree
      */
     DestObj->Package.Elements = ACPI_ALLOCATE_ZEROED (
-                                    ((ACPI_SIZE) SourceObj->Package.Count + 1) *
-                                    sizeof (void *));
+        ((ACPI_SIZE) SourceObj->Package.Count + 1) *
+        sizeof (void *));
     if (!DestObj->Package.Elements)
     {
         ACPI_ERROR ((AE_INFO, "Package allocation failure"));
@@ -1086,7 +1115,7 @@ AcpiUtCopyIpackageToIpackage (
      * This handles nested packages of arbitrary depth.
      */
     Status = AcpiUtWalkPackageTree (SourceObj, DestObj,
-                AcpiUtCopyIelementToIelement, WalkState);
+        AcpiUtCopyIelementToIelement, WalkState);
     if (ACPI_FAILURE (Status))
     {
         /* On failure, delete the destination package object */
@@ -1136,13 +1165,20 @@ AcpiUtCopyIobjectToIobject (
 
     if (SourceDesc->Common.Type == ACPI_TYPE_PACKAGE)
     {
-        Status = AcpiUtCopyIpackageToIpackage (SourceDesc, *DestDesc,
-                        WalkState);
+        Status = AcpiUtCopyIpackageToIpackage (
+            SourceDesc, *DestDesc, WalkState);
     }
     else
     {
         Status = AcpiUtCopySimpleObject (SourceDesc, *DestDesc);
     }
 
+    /* Delete the allocated object if copy failed */
+
+    if (ACPI_FAILURE (Status))
+    {
+        AcpiUtRemoveReference (*DestDesc);
+    }
+
     return_ACPI_STATUS (Status);
 }
