@@ -8,13 +8,13 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2011, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
  * All rights reserved.
  *
  * 2. License
  *
  * 2.1. This is your license from Intel Corp. under its intellectual property
- * rights.  You may have additional license terms from the party that provided
+ * rights. You may have additional license terms from the party that provided
  * you this software, covering your right to use that party's intellectual
  * property rights.
  *
@@ -31,7 +31,7 @@
  * offer to sell, and import the Covered Code and derivative works thereof
  * solely to the minimum extent necessary to exercise the above copyright
  * license, and in no event shall the patent license extend to any additions
- * to or modifications of the Original Intel Code.  No other license or right
+ * to or modifications of the Original Intel Code. No other license or right
  * is granted directly or by implication, estoppel or otherwise;
  *
  * The above copyright and patent license is granted only if the following
@@ -43,11 +43,11 @@
  * Redistribution of source code of any substantial portion of the Covered
  * Code or modification with rights to further distribute source must include
  * the above Copyright Notice, the above License, this list of Conditions,
- * and the following Disclaimer and Export Compliance provision.  In addition,
+ * and the following Disclaimer and Export Compliance provision. In addition,
  * Licensee must cause all Covered Code to which Licensee contributes to
  * contain a file documenting the changes Licensee made to create that Covered
- * Code and the date of any change.  Licensee must include in that file the
- * documentation of any changes made by any predecessor Licensee.  Licensee
+ * Code and the date of any change. Licensee must include in that file the
+ * documentation of any changes made by any predecessor Licensee. Licensee
  * must include a prominent statement that the modification is derived,
  * directly or indirectly, from Original Intel Code.
  *
@@ -55,7 +55,7 @@
  * Redistribution of source code of any substantial portion of the Covered
  * Code or modification without rights to further distribute source must
  * include the following Disclaimer and Export Compliance provision in the
- * documentation and/or other materials provided with distribution.  In
+ * documentation and/or other materials provided with distribution. In
  * addition, Licensee may not authorize further sublicense of source of any
  * portion of the Covered Code, and must include terms to the effect that the
  * license from Licensee to its licensee is limited to the intellectual
@@ -80,10 +80,10 @@
  * 4. Disclaimer and Export Compliance
  *
  * 4.1. INTEL MAKES NO WARRANTY OF ANY KIND REGARDING ANY SOFTWARE PROVIDED
- * HERE.  ANY SOFTWARE ORIGINATING FROM INTEL OR DERIVED FROM INTEL SOFTWARE
- * IS PROVIDED "AS IS," AND INTEL WILL NOT PROVIDE ANY SUPPORT,  ASSISTANCE,
- * INSTALLATION, TRAINING OR OTHER SERVICES.  INTEL WILL NOT PROVIDE ANY
- * UPDATES, ENHANCEMENTS OR EXTENSIONS.  INTEL SPECIFICALLY DISCLAIMS ANY
+ * HERE. ANY SOFTWARE ORIGINATING FROM INTEL OR DERIVED FROM INTEL SOFTWARE
+ * IS PROVIDED "AS IS," AND INTEL WILL NOT PROVIDE ANY SUPPORT, ASSISTANCE,
+ * INSTALLATION, TRAINING OR OTHER SERVICES. INTEL WILL NOT PROVIDE ANY
+ * UPDATES, ENHANCEMENTS OR EXTENSIONS. INTEL SPECIFICALLY DISCLAIMS ANY
  * IMPLIED WARRANTIES OF MERCHANTABILITY, NONINFRINGEMENT AND FITNESS FOR A
  * PARTICULAR PURPOSE.
  *
@@ -92,14 +92,14 @@
  * COSTS OF PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, OR FOR ANY INDIRECT,
  * SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THIS AGREEMENT, UNDER ANY
  * CAUSE OF ACTION OR THEORY OF LIABILITY, AND IRRESPECTIVE OF WHETHER INTEL
- * HAS ADVANCE NOTICE OF THE POSSIBILITY OF SUCH DAMAGES.  THESE LIMITATIONS
+ * HAS ADVANCE NOTICE OF THE POSSIBILITY OF SUCH DAMAGES. THESE LIMITATIONS
  * SHALL APPLY NOTWITHSTANDING THE FAILURE OF THE ESSENTIAL PURPOSE OF ANY
  * LIMITED REMEDY.
  *
  * 4.3. Licensee shall not export, either directly or indirectly, any of this
  * software or system incorporating such software without first obtaining any
  * required license or other approval from the U. S. Department of Commerce or
- * any other agency or department of the United States Government.  In the
+ * any other agency or department of the United States Government. In the
  * event Licensee exports any such software from the United States or
  * re-exports any such software from a foreign destination, Licensee shall
  * ensure that the distribution and export/re-export of the software is in
@@ -111,6 +111,42 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
 #ifndef __ACPIHELP_H
@@ -121,29 +157,77 @@
 #include "accommon.h"
 #include "acapps.h"
 
-#include <stdio.h>
-#include <sys/stat.h>
 #include <sys/types.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <string.h>
 #ifdef WIN32
 #include <io.h>
 #include <direct.h>
 #endif
-#include <errno.h>
 
 
-#define     AH_DECODE_DEFAULT           0
-#define     AH_DECODE_ASL               1
-#define     AH_DECODE_ASL_KEYWORD       2
-#define     AH_DECODE_PREDEFINED_NAME   3
-#define     AH_DECODE_AML               4
-#define     AH_DECODE_AML_OPCODE        5
+/*
+ * Global variables. Defined in ahmain.c only, externed in all other files
+ */
+#undef ACPI_GLOBAL
+#undef ACPI_INIT_GLOBAL
 
-#define     AH_MAX_ASL_LINE_LENGTH      70
-#define     AH_MAX_AML_LINE_LENGTH      100
+#ifdef DEFINE_AHELP_GLOBALS
+#define ACPI_GLOBAL(type,name) \
+    extern type name; \
+    type name
 
+#define ACPI_INIT_GLOBAL(type,name,value) \
+    type name=value
+
+#else
+#ifndef ACPI_GLOBAL
+#define ACPI_GLOBAL(type,name) \
+    extern type name
+#endif
+
+#ifndef ACPI_INIT_GLOBAL
+#define ACPI_INIT_GLOBAL(type,name,value) \
+    extern type name
+#endif
+#endif
+
+
+#define AH_BUFFER_LENGTH                128
+#define AH_LINE_BUFFER_LENGTH           512
+#define AH_MAX_ASL_LINE_LENGTH          70
+#define AH_MAX_AML_LINE_LENGTH          100
+
+ACPI_GLOBAL (char,                      Gbl_Buffer[AH_BUFFER_LENGTH]);
+ACPI_GLOBAL (char,                      Gbl_LineBuffer[AH_LINE_BUFFER_LENGTH]);
+extern const AH_PREDEFINED_NAME         AslPredefinedInfo[];
+extern const AH_DEVICE_ID               AslDeviceIds[];
+
+
+#define AH_DISPLAY_EXCEPTION(Status, Name) \
+    printf ("%.4X: %s\n", Status, Name)
+
+#define AH_DISPLAY_EXCEPTION_TEXT(Status, Exception) \
+    printf ("%.4X: %-28s (%s)\n", Status,\
+    Exception->Name, Exception->Description)
+
+
+typedef enum
+{
+    AH_DECODE_DEFAULT           = 0,
+    AH_DECODE_ASL,
+    AH_DECODE_ASL_KEYWORD,
+    AH_DECODE_PREDEFINED_NAME,
+    AH_DECODE_AML,
+    AH_DECODE_AML_OPCODE,
+    AH_DECODE_AML_TYPE,
+    AH_DECODE_ASL_AML,
+    AH_DECODE_EXCEPTION,
+
+    AH_DISPLAY_DEVICE_IDS,
+    AH_DISPLAY_UUIDS,
+    AH_DISPLAY_TABLES,
+    AH_DISPLAY_DIRECTIVES
+
+} AH_OPTION_TYPES;
 
 typedef struct ah_aml_opcode
 {
@@ -158,6 +242,13 @@ typedef struct ah_aml_opcode
 
 } AH_AML_OPCODE;
 
+typedef struct ah_aml_type
+{
+    char            *Name;
+    char            *Description;
+
+} AH_AML_TYPE;
+
 typedef struct ah_asl_operator
 {
     char            *Name;
@@ -174,24 +265,24 @@ typedef struct ah_asl_keyword
 
 } AH_ASL_KEYWORD;
 
-typedef struct ah_predefined_name
+typedef struct ah_directive_info
 {
     char            *Name;
     char            *Description;
-    char            *Action;
 
-} AH_PREDEFINED_NAME;
+} AH_DIRECTIVE_INFO;
 
 
-extern const AH_AML_OPCODE          AmlOpcodeInfo[];
-extern const AH_ASL_OPERATOR        AslOperatorInfo[];
-extern const AH_ASL_KEYWORD         AslKeywordInfo[];
-extern const AH_PREDEFINED_NAME     AslPredefinedInfo[];
-extern BOOLEAN                      AhDisplayAll;
+/* Externals for various data tables */
+
+extern const AH_AML_OPCODE          Gbl_AmlOpcodeInfo[];
+extern const AH_AML_TYPE            Gbl_AmlTypesInfo[];
+extern const AH_ASL_OPERATOR        Gbl_AslOperatorInfo[];
+extern const AH_ASL_KEYWORD         Gbl_AslKeywordInfo[];
+extern const AH_UUID                Gbl_AcpiUuids[];
+extern const AH_DIRECTIVE_INFO      Gbl_PreprocessorDirectives[];
+extern const AH_TABLE               AcpiGbl_SupportedTables[];
 
-void
-AhStrupr (
-    char                    *SrcString);
 
 void
 AhFindAmlOpcode (
@@ -201,11 +292,19 @@ void
 AhDecodeAmlOpcode (
     char                    *Name);
 
+void
+AhDecodeException (
+    char                    *Name);
+
 void
 AhFindPredefinedNames (
     char                    *Name);
 
 void
+AhFindAslAndAmlOperators (
+    char                    *Name);
+
+UINT32
 AhFindAslOperators (
     char                    *Name);
 
@@ -213,4 +312,35 @@ void
 AhFindAslKeywords (
     char                    *Name);
 
+void
+AhFindAmlTypes (
+    char                    *Name);
+
+void
+AhDisplayDeviceIds (
+    char                    *Name);
+
+void
+AhDisplayTables (
+    void);
+
+const AH_TABLE *
+AcpiAhGetTableInfo (
+    char                    *Signature);
+
+void
+AhDisplayUuids (
+    void);
+
+void
+AhDisplayDirectives (
+    void);
+
+void
+AhPrintOneField (
+    UINT32                  Indent,
+    UINT32                  CurrentPosition,
+    UINT32                  MaxPosition,
+    const char              *Field);
+
 #endif /* __ACPIHELP_H */
