@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2015, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -111,6 +111,42 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
 #include "acpisrc.h"
@@ -228,7 +264,6 @@ AsRemoveConditionalCompile (
 
 
     KeywordLength = strlen (Keyword);
-    SubBuffer = Buffer;
     SubString = Buffer;
 
     while (SubString)
@@ -365,7 +400,7 @@ AsRemoveConditionalCompile (
 
         /* Remove the lines */
 
-        SubBuffer = AsRemoveData (SubString, SubBuffer);
+        (void) AsRemoveData (SubString, SubBuffer);
     }
 }
 
@@ -514,6 +549,7 @@ AsReduceTypedefs (
 {
     char                    *SubString;
     char                    *SubBuffer;
+    char                    *SubSubString;
     int                     NestLevel;
 
 
@@ -526,56 +562,89 @@ AsReduceTypedefs (
 
         if (SubString)
         {
-            /* Remove the typedef itself */
+            SubSubString = SubString + strlen (Keyword);
 
-            SubBuffer = SubString + strlen ("typedef") + 1;
-            SubBuffer = AsRemoveData (SubString, SubBuffer);
+            /* skip spaces */
 
-            /* Find the opening brace of the struct or union */
-
-            while (*SubString != '{')
+            while (strchr(" \t\r\n", *SubSubString))
             {
-                SubString++;
+                SubSubString++;
             }
-            SubString++;
 
-            /* Find the closing brace. Handles nested braces */
+            /* skip type name */
 
-            NestLevel = 1;
-            while (*SubString)
+            while (!strchr(" \t\r\n", *SubSubString))
             {
-                if (*SubString == '{')
-                {
-                    NestLevel++;
-                }
-                else if (*SubString == '}')
-                {
-                    NestLevel--;
-                }
+                SubSubString++;
+            }
 
+            /* skip spaces */
+
+            while (strchr(" \t\r\n", *SubSubString))
+            {
+                SubSubString++;
+            }
+
+            if (*SubSubString == '{')
+            {
+                /* Remove the typedef itself */
+
+                SubBuffer = SubString + strlen ("typedef") + 1;
+                (void) AsRemoveData (SubString, SubBuffer);
+
+                /* Find the opening brace of the struct or union */
+
+                while (*SubString != '{')
+                {
+                    SubString++;
+                }
                 SubString++;
 
-                if (NestLevel == 0)
+                /* Find the closing brace. Handles nested braces */
+
+                NestLevel = 1;
+                while (*SubString)
                 {
-                    break;
+                    if (*SubString == '{')
+                    {
+                        NestLevel++;
+                    }
+                    else if (*SubString == '}')
+                    {
+                        NestLevel--;
+                    }
+
+                    SubString++;
+
+                    if (NestLevel == 0)
+                    {
+                        break;
+                    }
                 }
+
+                /* Remove an extra line feed if present */
+
+                if (!strncmp (SubString - 3, "\n\n", 2))
+                {
+                    *(SubString -2) = '}';
+                    SubString--;
+                }
+
+                /* Find the end of the typedef name */
+
+                SubBuffer = AsSkipUntilChar (SubString, ';');
+
+                /* And remove the typedef name */
+
+                SubBuffer = AsRemoveData (SubString, SubBuffer);
             }
-
-            /* Remove an extra line feed if present */
-
-            if (!strncmp (SubString - 3, "\n\n", 2))
+            else
             {
-                *(SubString -2) = '}';
-                SubString--;
+                /* Skip the entire definition */
+
+                SubString = strchr (SubString, ';') + 1;
+                SubBuffer = SubString;
             }
-
-            /* Find the end of the typedef name */
-
-            SubBuffer = AsSkipUntilChar (SubString, ';');
-
-            /* And remove the typedef name */
-
-            SubBuffer = AsRemoveData (SubString, SubBuffer);
         }
     }
 }
@@ -689,6 +758,7 @@ AsRemoveDebugMacros (
 
     AsReplaceString ("return_VOID",         "return", REPLACE_WHOLE_WORD, Buffer);
     AsReplaceString ("return_PTR",          "return", REPLACE_WHOLE_WORD, Buffer);
+    AsReplaceString ("return_STR",          "return", REPLACE_WHOLE_WORD, Buffer);
     AsReplaceString ("return_ACPI_STATUS",  "return", REPLACE_WHOLE_WORD, Buffer);
     AsReplaceString ("return_acpi_status",  "return", REPLACE_WHOLE_WORD, Buffer);
     AsReplaceString ("return_VALUE",        "return", REPLACE_WHOLE_WORD, Buffer);
