@@ -7,10 +7,14 @@
 postfix=`date +%H%M%S`
 tmp_iasl=/tmp/iasl-$postfix
 tmp_acpiexec=/tmp/acpiexec-$postfix
+tmp_acpibin=/tmp/acpibin-$postfix
 
 TEST_CASES=
 TEST_MODES=
 REBUILD_TOOLS=yes
+BINCOMPONLY=no
+DATATABLEONLY=no
+EXECONLY=no
 
 usage() {
 
@@ -20,13 +24,16 @@ usage() {
 	echo "  -c:	Specify individual test cases (can be used multiple times)"
 	echo "  -m:	Specify individual test modes (can be used multiple times)"
 	echo "  -u:	Do not force rebuilding of ACPICA utilities (acpiexec, iasl)"
+	echo "  -e:     Perform the execution of aml files and omit binary comparison of regular aml and disassembled aml file."
+	echo "  -b:     Only perform binary comparison of regular aml and disasssembled aml file"
+	echo "  -d:     Only execute data table compiler/disassembler test"
 	echo ""
 
 	echo "Available test modes:"
-	echo "  n32	32-bit normal mode"
-	echo "  n64	64-bit normal mode"
-	echo "  s32	32-bit slack mode"
-	echo "  s64	64-bit slack mode"
+	echo "  n32	32-bit unoptimized code (tests are compiled with iasl -oa -r 1 and other flags)"
+	echo "  n64	64-bit unoptimized code (tests are compiled with iasl -oa -r 2 and other flags)"
+	echo "  o32	32-bit optimized code (tests are compiled with iasl -r 1 and other flags)"
+	echo "  o64	64-bit optimized code (tests are compiled with iasl -r 2 and other flags)"
 	echo ""
 
 	Do 3
@@ -61,6 +68,7 @@ setup_environment() {
 
 	export ASL=$tmp_iasl
 	export acpiexec=$tmp_acpiexec		
+	export acpibin=$tmp_acpibin
 	export ASLTSDIR=$aslts_dir
 	export PATH=$ASLTSDIR/bin:$PATH
 }
@@ -71,22 +79,25 @@ build_acpi_tools() {
 
 	restore_dir=$PWD
 	cd ${generation_dir}
-	rm -f $tmp_iasl $tmp_acpiexec
+	rm -f $tmp_iasl $tmp_acpiexec $tmp_acpibin
 
 	# Build native-width iASL compiler and acpiexec
 	if [ ! -e bin/iasl -o ! -e bin/acpiexec ]; then
 		REBUILD_TOOLS=yes
 	fi
 	if [ "x$REBUILD_TOOLS" = "xyes" ]; then
+		jobs=`nproc`
 		make clean
-		make iasl ASLTS=TRUE
-		make acpiexec ASLTS=TRUE
+		make iasl ASLTS=TRUE -j$jobs
+		make acpibin ASLTS=TRUE -j$jobs
+		make acpiexec ASLTS=TRUE -j$jobs
 	fi
 
 	if [ -d "bin" ] && [ -f "bin/iasl" ]; then
 		echo "Installing ACPICA tools"
 		cp bin/iasl $tmp_iasl
 		cp bin/acpiexec $tmp_acpiexec
+		cp bin/acpibin $tmp_acpibin
 	else
 		echo "Could not find iASL/acpiexec tools"
 		exit
@@ -99,11 +110,37 @@ build_acpi_tools() {
 	elif [ ! -f $tmp_acpiexec ] ; then
 		echo "acpiexec utility not found"
 		exit
+	elif [ ! -f $tmp_acpibin ] ; then
+		echo "acpibin utility not found"
+		exit
 	fi
 
 	cd $restore_dir
 }
 
+# Run a simple compiler test.
+# This test does the following:
+# 1 generate all sample tables in the compiler
+# 2 compile all tables (.asl -> .aml)
+# 3 disassembles all tables (.aml -> .dsl)
+# 4 recompiles all all tables (.dsl -> recomp.aml)
+# 5 runs binary comparison between .aml and recomp.aml
+run_compiler_template_test()
+{
+	pushd templates
+
+	rm -f *.asl *.aml *.dsl
+
+	$ASL -T all 2> /dev/null
+	for filename in *.asl
+	do
+		make -s NAME=$(basename "$filename" .asl)
+	done
+
+	rm -f *.asl *.aml *.dsl
+	popd
+}
+
 
 # Compile and run the ASLTS suite
 run_aslts() {
@@ -112,32 +149,40 @@ run_aslts() {
 	version=`$ASL | grep version | awk '{print $5}'`
 	rm -rf $ASLTSDIR/tmp/aml/$version
 
-	if [ "x$TEST_CASES" = "x" ]; then
-		# Compile all ASL test modules
-		Do 0 aslts
-		if [ $? -ne 0 ]; then
-			echo "ASLTS Compile Failure"
-			exit 1
-		fi
-	else
-		Do 0 $TEST_CASES
+	# run templates test
+
+	run_compiler_template_test
+
+	if [ "x$DATATABLEONLY" = "xyes" ]; then
+		return 0
+	fi;
+
+	if [ "x$TEST_MODES" = "x" ]; then
+		TEST_MODES="n32 n64 o32 o64"
+	fi
+	Do 0 $TEST_MODES $TEST_CASES $EXECONLY
+	if [ $? -ne 0 ]; then
+		echo "ASLTS Compile Failure"
+		exit 1
 	fi
 
 	# Execute the test suite
-	echo ""
-	echo "ASL Test Suite Started: `date`"
-	start_time=$(date)
+	if [ "x$BINCOMPONLY" = "xno" ]; then
+		echo ""
+		echo "ASL Test Suite Started: `date`"
+		start_time=$(date)
 
-	if [ "x$TEST_MODES" = "x" ]; then
-		TEST_MODES="n32 n64 s32 s64"
-	fi
-	Do 1 $TEST_MODES $TEST_CASES
+		if [ "x$TEST_MODES" = "x" ]; then
+			TEST_MODES="n32 n64 o32 o64"
+		fi
+		Do 1 $TEST_MODES $TEST_CASES
 
-	echo ""
-	echo "ASL Test Suite Finished: `date`"
-	echo "                Started: $start_time"
+		echo ""
+		echo "ASL Test Suite Finished: `date`"
+		echo "                Started: $start_time"
 
-	rm -f $tmp_iasl $tmp_acpiexec
+		rm -f $tmp_iasl $tmp_acpiexec $tmp_acpibin
+	fi;
 }
 
 SRCDIR=`(cd \`dirname $0\`; cd ..; pwd)`
@@ -150,9 +195,13 @@ RESET_SETTINGS
 INIT_ALL_AVAILABLE_CASES
 INIT_ALL_AVAILABLE_MODES
 
-while getopts "c:m:u" opt
+while getopts "c:m:uebd" opt
 do
 	case $opt in
+	b)
+		BINCOMPONLY=yes
+		echo "Running only binary comparisons"
+	;;
 	c)
 		get_collection_opcode "$OPTARG"
 		if [ $? -eq $COLLS_NUM ]; then
@@ -162,6 +211,14 @@ do
 			TEST_CASES="$OPTARG $TEST_CASES"
 		fi
 	;;
+	d)
+		DATATABLEONLY=yes
+		echo "Running only data table test"
+	;;
+	e)
+		EXECONLY=yes
+		echo "Running tests without binary comparisons"
+	;;
 	m)
 		check_mode_id "$OPTARG"
 		if [ $? -eq 1 ]; then
